"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const listr_1 = __importDefault(require("listr"));
const api_1 = __importDefault(require("@mesg/api"));
const orchestrator_1 = __importDefault(require("@mesg/orchestrator"));
const chalk_1 = __importDefault(require("chalk"));
const encoder_1 = require("@mesg/orchestrator/lib/encoder");
const docker_1 = require("../../utils/docker");
const Environment = __importStar(require("../../utils/environment-tasks"));
const Service = __importStar(require("../../utils/service"));
const Runner = __importStar(require("../../utils/runner"));
const execution_1 = require("@mesg/orchestrator/lib/execution");
const version_1 = __importDefault(require("../../version"));
const sign_1 = __importDefault(require("../../utils/sign"));
const ipfsClient = require('ipfs-http-client');
class Dev extends command_1.Command {
    constructor() {
        super(...arguments);
        this.lcdEndpoint = 'http://localhost:1317';
        this.orchestratorEndpoint = 'localhost:50052';
        this.lcd = new api_1.default(this.lcdEndpoint);
        this.orchestrator = new orchestrator_1.default(this.orchestratorEndpoint);
        this.ipfsClient = ipfsClient('ipfs.app.mesg.com', '5001', { protocol: 'http' });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args, flags } = this.parse(Dev);
            let definition;
            let service;
            let runner;
            const tasks = new listr_1.default([
                Environment.start,
                {
                    title: 'Compiling service',
                    task: () => __awaiter(this, void 0, void 0, function* () {
                        definition = yield Service.compile(args.PATH, this.ipfsClient);
                    })
                },
                {
                    title: 'Creating service',
                    task: (ctx) => __awaiter(this, void 0, void 0, function* () {
                        service = yield Service.create(this.lcd, definition, ctx.config.mnemonic);
                    })
                },
                {
                    title: 'Starting service',
                    task: (ctx) => __awaiter(this, void 0, void 0, function* () {
                        runner = yield Runner.create(this.lcdEndpoint, this.orchestratorEndpoint, ctx.config.mnemonic, ctx.engineAddress, service.hash, flags.env);
                    })
                },
                {
                    title: 'Fetching service\'s logs',
                    task: () => new listr_1.default([
                        {
                            title: 'Fetching service\'s logs',
                            task: () => __awaiter(this, void 0, void 0, function* () {
                                const dockerContainers = yield docker_1.listContainers({ label: [`mesg.runner=${runner.hash}`] });
                                this.logs = yield Promise.all(dockerContainers.map(x => x.logs({
                                    stderr: true,
                                    stdout: true,
                                    follow: true,
                                    tail: 'all',
                                })));
                            })
                        },
                        {
                            title: 'Fetching events\' logs',
                            task: ctx => {
                                const payload = {
                                    filter: {
                                        instanceHash: runner.instanceHash
                                    }
                                };
                                this.events = this.orchestrator.event.stream(payload, sign_1.default(payload, ctx.config.mnemonic));
                            }
                        },
                        {
                            title: 'Fetching executions\' logs',
                            task: (ctx) => __awaiter(this, void 0, void 0, function* () {
                                const payload = {
                                    filter: {
                                        executorHash: runner.hash,
                                        statuses: [
                                            execution_1.Status.Completed,
                                            execution_1.Status.Failed,
                                        ]
                                    }
                                };
                                this.results = this.orchestrator.execution.stream(payload, sign_1.default(payload, ctx.config.mnemonic));
                            })
                        }
                    ])
                }
            ]);
            const { config, engineAddress } = yield tasks.run({
                configDir: this.config.dataDir,
                pull: flags.pull,
                version: flags.version,
                endpoint: this.lcdEndpoint,
            });
            for (const log of this.logs) {
                log
                    .on('data', buffer => docker_1.parseLog(buffer).forEach(x => this.log(chalk_1.default.gray(x))))
                    .on('error', error => { this.warn('Docker log stream error: ' + error.message); });
            }
            this.events
                .on('data', event => this.log(`EVENT[${event.key}]: ` + chalk_1.default.gray(JSON.stringify(encoder_1.decode(event.data)))))
                .on('error', error => { this.warn('Event stream error: ' + error.message); });
            this.results
                .on('data', execution => execution.error
                ? this.log(`RESULT[${execution.taskKey}]: ` + chalk_1.default.red('ERROR:', execution.error))
                : this.log(`RESULT[${execution.taskKey}]: ` + chalk_1.default.gray(JSON.stringify(encoder_1.decode(execution.outputs)))))
                .on('error', error => { this.warn('Result stream error: ' + error.message); });
            process.once('SIGINT', () => __awaiter(this, void 0, void 0, function* () {
                yield new listr_1.default([
                    {
                        title: 'Stopping logs',
                        task: () => __awaiter(this, void 0, void 0, function* () {
                            if (this.logs)
                                this.logs.forEach((x) => x.destroy());
                            if (this.events)
                                this.events.cancel();
                            if (this.results)
                                this.results.cancel();
                            return Promise.resolve();
                        })
                    },
                    {
                        title: 'Stopping service',
                        skip: () => !service && !runner,
                        task: () => __awaiter(this, void 0, void 0, function* () {
                            return Runner.stop(this.lcdEndpoint, this.orchestratorEndpoint, config.mnemonic, engineAddress, runner.hash);
                        })
                    },
                    Environment.stop,
                ]).run({
                    configDir: this.config.dataDir
                });
            }));
        });
    }
}
exports.default = Dev;
Dev.description = 'Run a service in a local development environment';
Dev.flags = {
    version: command_1.flags.string({ name: 'Engine version', default: version_1.default.engine }),
    pull: command_1.flags.boolean({ name: 'Force to pull the docker image', default: false }),
    env: command_1.flags.string({
        description: 'Environment variables to inject to the service',
        multiple: true,
        helpValue: 'FOO=BAR'
    })
};
Dev.args = [{
        name: 'PATH',
        description: 'Path or url of a service',
        default: './'
    }];
