"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const fs_1 = require("fs");
const path_1 = require("path");
const Service = __importStar(require("../../utils/service"));
const handlebars_1 = require("handlebars");
const ipfsClient = require('ipfs-http-client');
class Doc extends command_1.Command {
    constructor() {
        super(...arguments);
        this.ipfsClient = ipfsClient('ipfs.app.mesg.com', '5001', { protocol: 'http' });
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args, flags } = this.parse(Doc);
            const definition = yield Service.compile(args.SERVICE_PATH, this.ipfsClient);
            handlebars_1.registerHelper('or', (a, b) => a ? a : b);
            handlebars_1.registerHelper('toJSON', JSON.stringify);
            const template = fs_1.readFileSync(path_1.join(__dirname, '..', 'assets', 'doc.md')).toString();
            const markdown = handlebars_1.compile(template)(definition);
            if (flags.save) {
                const defaultReadmeFileName = fs_1.readdirSync(args.SERVICE_PATH).find(file => {
                    return /^readme(?:.(?:md|txt)+)?$/i.test(file);
                });
                const readmeFileName = defaultReadmeFileName || 'README.md';
                fs_1.writeFileSync(path_1.join(args.SERVICE_PATH, readmeFileName), markdown);
            }
            else {
                this.log(markdown);
            }
            return markdown;
        });
    }
}
exports.default = Doc;
Doc.description = 'Generate documentation for service and print it to stdout';
Doc.flags = {
    save: command_1.flags.boolean({ char: 's', description: 'Save to default readme file' }),
};
Doc.args = [{
        name: 'SERVICE_PATH',
        description: 'Path of a service',
        default: './'
    }];
