"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@oclif/command");
const axios_1 = __importDefault(require("axios"));
const fs_1 = require("fs");
const inquirer_1 = require("inquirer");
const path_1 = require("path");
const deployer_1 = __importDefault(require("../../utils/deployer"));
const cli_ux_1 = require("cli-ux");
const templatesURL = 'https://raw.githubusercontent.com/mesg-foundation/awesome/master/templates.json';
class Init extends command_1.Command {
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            const { args, flags } = this.parse(Init);
            const templateUrl = yield this.getTemplateUrl(flags.template);
            cli_ux_1.cli.action.start('Initializing your project');
            yield this.downloadTemplate(args.DIR, templateUrl);
            cli_ux_1.cli.action.stop(args.DIR);
            return args.DIR;
        });
    }
    getTemplateUrl(template) {
        return __awaiter(this, void 0, void 0, function* () {
            if (template) {
                return template;
            }
            cli_ux_1.cli.action.start('Fetching templates');
            const templates = yield this.fetchTemplates();
            cli_ux_1.cli.action.stop();
            const { value } = (yield inquirer_1.prompt({
                type: 'list',
                name: 'value',
                message: 'Select the template to use',
                default: 'Basic',
                choices: templates.map(x => ({
                    name: `${x.name} ➜ ${x.url}`,
                    value: x.name,
                    short: x.name
                }))
            }));
            const selectedTemplate = templates.find(x => x.name === value);
            if (!selectedTemplate) {
                throw new Error(`The template name "${value}" is not valid`);
            }
            return selectedTemplate.url;
        });
    }
    fetchTemplates() {
        return __awaiter(this, void 0, void 0, function* () {
            const response = yield axios_1.default.get(templatesURL);
            return response.data;
        });
    }
    downloadTemplate(path, url) {
        return __awaiter(this, void 0, void 0, function* () {
            const tmpPath = yield deployer_1.default(url);
            fs_1.renameSync(path_1.join(tmpPath, 'template'), path);
        });
    }
}
exports.default = Init;
Init.description = 'Initialize a service from a template';
Init.flags = {
    template: command_1.flags.string({ char: 't', description: 'Specify the template URL to use' }),
};
Init.args = [{
        name: 'DIR',
        required: true,
        description: 'Directory to initialize a service into'
    }];
