"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const js_yaml_1 = require("js-yaml");
const path_1 = require("path");
const bech32_1 = require("bech32");
const account_1 = __importStar(require("@mesg/api/lib/account"));
const ENGINE_FILE = 'config.yml';
const CLI_FILE = '.mesgrc';
exports.clear = (path) => {
    const rm = fs_1.rmdirSync;
    rm(path_1.join(path, 'cosmos'), { recursive: true });
    rm(path_1.join(path, 'tendermint'), { recursive: true });
    fs_1.unlinkSync(path_1.join(path, ENGINE_FILE));
    fs_1.unlinkSync(path_1.join(path, CLI_FILE));
};
const read = (filepath) => {
    return fs_1.existsSync(filepath)
        ? js_yaml_1.safeLoad(fs_1.readFileSync(filepath).toString())
        : {};
};
const write = (path, config) => {
    fs_1.writeFileSync(path_1.join(path, CLI_FILE), js_yaml_1.safeDump(config));
    fs_1.writeFileSync(path_1.join(path, ENGINE_FILE), js_yaml_1.safeDump(config.engine));
    return config;
};
const hasTestAccount = (path) => {
    if (!fs_1.existsSync(path_1.join(path, ENGINE_FILE)))
        return false;
    if (!fs_1.existsSync(path_1.join(path, CLI_FILE)))
        return false;
    const config = read(path_1.join(path, CLI_FILE));
    return !!config.mnemonic && !!config.engine.account.mnemonic;
};
// https://github.com/forbole/big-dipper/blob/master/imports/startup/server/util.js
const pubKey = (mnemonic) => {
    const publicKey = account_1.default.deriveMnemonic(mnemonic).publicKey;
    const pubkeyAminoPrefix = Buffer.from('EB5AE98721', 'hex');
    const buffer = Buffer.alloc(38);
    pubkeyAminoPrefix.copy(buffer, 0);
    Buffer.from(publicKey).copy(buffer, pubkeyAminoPrefix.length);
    return bech32_1.encode(`${account_1.bech32Prefix}pub`, bech32_1.toWords(buffer));
};
exports.generateConfig = (path) => {
    if (hasTestAccount(path))
        return read(path_1.join(path, CLI_FILE));
    const mnemonic = account_1.default.generateMnemonic();
    const config = {
        engine: {
            authorized_pubkeys: [
                pubKey(mnemonic)
            ],
            account: {
                mnemonic: account_1.default.generateMnemonic()
            }
        },
        mnemonic
    };
    return write(path, config);
};
