"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs_1 = require("fs");
const hosted_git_info_1 = require("hosted-git-info");
const ignore_1 = __importDefault(require("ignore"));
const is_git_url_1 = __importDefault(require("is-git-url"));
const os_1 = require("os");
const path_1 = require("path");
const rimraf_1 = require("rimraf");
const tar_1 = __importDefault(require("tar"));
const uuid_1 = require("uuid");
const validator_1 = require("validator");
const MB = 1024 * 1024;
const MAX_UPLOAD_SIZE = MB * 10;
const downloadTarball = require('download-tarball');
const gitclone = require('git-clone');
const downloadSrc = (url) => __awaiter(void 0, void 0, void 0, function* () {
    const dir = path_1.join(os_1.tmpdir(), uuid_1.v4());
    if (hosted_git_info_1.fromUrl(url)) {
        return clone(url, dir);
    }
    yield downloadTarball({ url, dir });
    return dir;
});
const clone = (git, dir) => __awaiter(void 0, void 0, void 0, function* () {
    const url = git.split('#')[0];
    const checkout = git.split('#')[1] || 'master';
    return new Promise((resolve, reject) => {
        const opts = {
            checkout,
            shallow: checkout === 'master'
        };
        gitclone(url, dir, opts, (err) => {
            if (err)
                return reject(err);
            rimraf_1.sync(path_1.join(dir, '.git'));
            return resolve(dir);
        });
    });
});
const preprocessPath = (path) => {
    const directories = fs_1.readdirSync(path)
        .map(name => path_1.join(path, name))
        .filter(x => fs_1.lstatSync(x).isDirectory());
    if (fs_1.readdirSync(path).length === 1 && directories.length === 1) {
        return directories[0];
    }
    return path;
};
const directorySize = (path, accept) => {
    const fileAndStat = (x) => ({ file: x, stat: fs_1.statSync(path_1.join(path, x)) });
    const sizeOf = ({ file, stat }) => stat.isDirectory()
        ? directorySize(path_1.join(path, file), accept)
        : stat.size;
    return fs_1.readdirSync(path, 'utf8')
        .filter(accept)
        .map(fileAndStat)
        .map(sizeOf)
        .reduce((prev, next) => prev + next, 0);
};
exports.createTar = (path) => {
    const mesgignore = path_1.join(path, '.mesgignore');
    const ig = ignore_1.default().add([
        '.git',
        '.mesg',
        ...(fs_1.existsSync(mesgignore) ? fs_1.readFileSync(mesgignore).toString().split('\n') : [])
    ]);
    const filter = ig.createFilter();
    const dirSize = directorySize(path, filter);
    if (dirSize > MAX_UPLOAD_SIZE)
        throw new Error(`Max size upload ${MAX_UPLOAD_SIZE / MB}MB`);
    return tar_1.default.create({
        cwd: path,
        filter,
        strict: true,
        gzip: true,
        portable: true,
        noMtime: true,
    }, fs_1.readdirSync(path))
        .on('error', (error) => {
        throw error;
    });
};
exports.default = (pathOrUrl) => __awaiter(void 0, void 0, void 0, function* () {
    if (is_git_url_1.default(pathOrUrl) || validator_1.isURL(pathOrUrl)) {
        return preprocessPath(yield downloadSrc(pathOrUrl));
    }
    return preprocessPath(pathOrUrl);
});
