"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const node_docker_api_1 = require("node-docker-api");
const container_1 = __importDefault(require("@mesg/runner/lib/providers/container/container"));
const defaultClient = new node_docker_api_1.Docker(null);
exports.engineLabel = "mesg.engine=true";
exports.engineLabelMap = { [exports.engineLabel.split("=")[0]]: exports.engineLabel.split("=")[1] };
exports.engineName = "mesg_engine";
exports.parseLog = (buffer) => {
    return buffer.toString()
        .split('\n')
        .map(x => x.substring(8)) // Skip the 8 characters that docker put in front of its logs
        .filter(x => x);
};
exports.hasImage = (image, client = defaultClient) => __awaiter(void 0, void 0, void 0, function* () {
    return container_1.default.hasImage(client, image);
});
exports.fetchImageTag = (image, tag = 'latest', client = defaultClient) => __awaiter(void 0, void 0, void 0, function* () {
    return container_1.default.fetchImage(client, `${image}:${tag}`);
});
exports.findNetwork = (name, client = defaultClient) => __awaiter(void 0, void 0, void 0, function* () {
    const networks = yield client.network.list({
        filters: { name: [name] }
    });
    return networks[0];
});
exports.createNetwork = (client = defaultClient) => __awaiter(void 0, void 0, void 0, function* () {
    const network = yield exports.findNetwork(exports.engineName, client);
    if (network)
        return network;
    return client.network.create({
        CheckDuplicate: true,
        Name: exports.engineName,
        labels: exports.engineLabelMap
    });
});
exports.createContainer = (image, directory, client = defaultClient) => __awaiter(void 0, void 0, void 0, function* () {
    const network = yield exports.createNetwork(client);
    const container = new container_1.default({
        Labels: exports.engineLabelMap,
        Image: image,
        HostConfig: {
            RestartPolicy: {
                Name: 'on-failure'
            },
            Mounts: [{
                    Source: '/var/run/docker.sock',
                    Target: '/var/run/docker.sock',
                    Type: 'bind',
                }, {
                    Source: directory,
                    Target: '/root/.mesg',
                    Type: 'bind',
                }]
        }
    }, exports.engineName);
    container.addPorts(['1317', '50052']);
    container.connectTo(network, ["engine"]);
    return container.start();
});
exports.listContainers = (filters, client = defaultClient) => __awaiter(void 0, void 0, void 0, function* () {
    // docker service ls --filter doesn't do an exact match https://github.com/moby/moby/issues/32985
    const res = yield client.container.list({ filters });
    return filters.name
        ? res.filter(x => x.data['Name'] === filters.name[0])
        : res;
});
exports.logs = (name, follow, tail) => __awaiter(void 0, void 0, void 0, function* () {
    const containers = yield exports.listContainers({ name: [name] });
    if (containers.length === 0) {
        throw new Error(`No containers ${name} found.`);
    }
    return (containers[0]).logs({
        stderr: true,
        stdout: true,
        follow,
        tail
    });
});
